/*
 * BackgroundLoader.java
 *
 * DO NOT MODIFY THIS CODE
 */

package ex1;

import java.util.concurrent.*;

/**
 * Creates a background workload for the system using multiple threads to create
 * and exchange objects, generate garbage and so induce garbage collection
 *
 */
public class BackgroundLoader {
    
    private static final int NTHREADS = Runtime.getRuntime().availableProcessors() *2;
    private final ExecutorService pool = Executors.newFixedThreadPool(NTHREADS);
    private final Exchanger<Object[]> exchanger = new Exchanger<Object[]>();
    private final Runnable exchangeTask = new Runnable() {
            public void run() {
                while (!Thread.interrupted()) {
                    // allocate
                    Object[] arr = new Object[1024];
                    for (int i = 0; i < arr.length; i++) {
                        arr[i] = new Object();
                    }
                    // exchange
                    try {
                        arr = exchanger.exchange(arr);
                    }
                    catch (InterruptedException ex) {
                        break;
                    }
                    // make garbage
                    for (int i = 0 ; i < arr.length; i++) {
                        arr[i] = null;
                    }
                }
            }
        };
    
    /** Creates a new instance of BackgroundLoader */
    public BackgroundLoader() {
    }
    
    public void start() {
        for (int i = 0; i < NTHREADS; i++) {
            pool.execute(exchangeTask);
        }
    }
    
    public void shutdown() {
        pool.shutdownNow();
        try {
            pool.awaitTermination(5, TimeUnit.SECONDS);
        }
        catch (InterruptedException ex) {
            // ignore
        }
    }

}
