/*
 * PeriodicNHRT.java
 *
 */

package ex2;
import javax.realtime.*;
/**
 *
 * Skeleton class for a periodic NoHeapRealtimeThread
 */
public class PeriodicNHRT extends NoHeapRealtimeThread {
    
    public PeriodicNHRT(PriorityParameters scheduling, PeriodicParameters release, MemoryArea area) {
        super(scheduling, release, area);
        setName("PeriodicNHRT");
    }

    public void run() {
        int iterations = 100;
        Clock rtc = Clock.getRealtimeClock();
        // pre-create three time objects to avoid the need for allocation in the loop
        AbsoluteTime now = new AbsoluteTime(0,0);      // filled in with the current time
        AbsoluteTime lastTime = new AbsoluteTime(0,0); // a copy of the last value read into 'now'
        RelativeTime elapsed = new RelativeTime(0,0);  // holds the calculated elapsed time      
        now = rtc.getTime(now);
        lastTime.set(now);                          
        for (int i = 0; i < iterations; i++) {
            if (!RealtimeThread.waitForNextPeriod()) {
                throw new Error("Deadline missed!")   ;
            }
            lastTime.set(now);              
            now = rtc.getTime(now);             
            elapsed = now.subtract(lastTime, elapsed);
            // use seperate print calls to avoid string concatenation
            System.out.print("Elapsed time is ");            
            System.out.println(elapsed);            
        }
    }
}
