package ex4;
import java.util.Vector;
/**
 * The data communications class used to buffer the data sourced by the real-time
 * system activity, and consumed by non-realtime system activity.
 * The details of this class are not important for the purposes of the current
 * exercise.
 */

// DO NOT EDIT THIS FILE

public class DataBuffer {
    final Vector<Double> dataset;

     public DataBuffer() {
        dataset = new Vector<Double>(10000);
        // pre-populate to simulate an initial, partial data set
        for (int i = 0; i < 1000; i++) {
            dataset.add(new Double(Math.random()));
        }
        process();
     }

     /** Add the given value to the data buffer
      *
      * @param d the data to add
      */
     public void add(Double d) {
        dataset.add(d);
     }

     /**
      * Remove and return the head element in the data set
      * @return the head element, or null if the data set is empty
      */
     public Double take() {
         synchronized(dataset) {
             if (dataset.size() > 0) {
                 process(); // simulate data processing activity
                 return dataset.remove(0);
             }
         }
         return null;
     }

     /**
      * Simulates a data-processing operation - in this case a simple and
      * slow bubble-sort
      */
     private void process() {
         boolean sorted;
         do {
             sorted = true;
             for (int i = 0; i < dataset.size() - 1; i++) {
                 if (dataset.get(i).compareTo(dataset.get(i + 1)) < 0) {
                     Double d1 = dataset.get(i);
                     Double d2 = dataset.get(i + 1);
                     dataset.set(i, d2);
                     dataset.set(i + 1, d1);
                     sorted = false;
                 }
             }
         } while (!sorted);
     }
}
