/*
 * PeriodicAEH.java
 *
 */

package ex4;
import javax.realtime.*;
/**
 *
 * Skeleton class for a periodic NoHeap AsyncEventHandler
 */
public class PeriodicAEH extends AsyncEventHandler {

    /** The DataBuffer that we push our data to */
    private final DataBuffer buffer;

    /** The sensor we obtain data from */
    private final Sensor sensor;

    /**
     * Constructs a no-heap AsyncEventHandler using ImmortalMemory and the supplied
     * scheduing and release parameters.
     * @param scheduling the SchedulingParameters characterising this handler
     * @param release the ReleaseParameters characterising this handler
     * @param buffer the data buffer to push data to
     * @param sensor the sensor we read data from
     */
    public PeriodicAEH(PriorityParameters scheduling, PeriodicParameters release,
            DataBuffer buffer, Sensor sensor) {
        // DO NOT EDIT THIS CODE
        super(scheduling, release, null, ImmortalMemory.instance(), null, true, null);
        this.buffer = buffer;
        this.sensor = sensor;
        this.setDaemon(false);
    }

    int count = 0;

    public void handleAsyncEvent() {
        // INSERT CODE: to check for a missed deadline
        int pending = getAndClearPendingFireCount();
        if (pending > 0) {
            throw new Error("Deadline missed! - pending count " + pending);
        }

        // INSERT CODE: to read from sensor and add to buffer, and print
        Double data = sensor.read();
        buffer.add(data);
        System.out.printf("Handler read %.6f%n", data);

        // INSERT CODE: to terminate on the 100th sample
        if (count++ == 99)
            System.exit(0);
    }
}
